@@ -53,7 +53,7 @@ strtol(
      */
 
     p = string;
-    while (isspace(UCHAR(*p))) {
+    while (TclIsSpaceProc(*p)) {
 	p += 1;
     }
 
