@@ -17,15 +17,7 @@
 #include "tclInt.h"
 #include <dde.h>
 #include <ddeml.h>
-
-#ifndef UNICODE
-#   undef CP_WINUNICODE
-#   define CP_WINUNICODE CP_WINANSI
-#   undef Tcl_WinTCharToUtf
-#   define Tcl_WinTCharToUtf(a,b,c) Tcl_ExternalToUtfDString(NULL,a,b,c)
-#   undef Tcl_WinUtfToTChar
-#   define Tcl_WinUtfToTChar(a,b,c) Tcl_UtfToExternalDString(NULL,a,b,c)
-#endif
+#include <tchar.h>
 
 #if !defined(NDEBUG)
     /* test POKE server Implemented for debug mode only */
@@ -33,16 +25,6 @@
 #   define CBF_FAIL_POKES 0
 #endif
 
-/*
- * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the Dde_Init
- * declaration is in the source file itself, which is only accessed when we
- * are building a library. DO NOT MOVE BEFORE ANY #include LINES. ONLY USE
- * EXTERN TO INDICATE EXPORTED FUNCTIONS FROM NOW ON.
- */
-
-#undef TCL_STORAGE_CLASS
-#define TCL_STORAGE_CLASS DLLEXPORT
-
 /*
  * The following structure is used to keep track of the interpreters
  * registered by this process.
@@ -52,7 +34,7 @@ typedef struct RegisteredInterp {
     struct RegisteredInterp *nextPtr;
 				/* The next interp this application knows
 				 * about. */
-    TCHAR *name;			/* Interpreter's name (malloc-ed). */
+    WCHAR *name;		/* Interpreter's name (malloc-ed). */
     Tcl_Obj *handlerPtr;	/* The server handler command */
     Tcl_Interp *interp;		/* The interpreter attached to this name. */
 } RegisteredInterp;
@@ -69,7 +51,7 @@ typedef struct Conversation {
     Tcl_Obj *returnPackagePtr;	/* The result package for this conversation. */
 } Conversation;
 
-typedef struct DdeEnumServices {
+typedef struct {
     Tcl_Interp *interp;
     int result;
     ATOM service;
@@ -77,7 +59,7 @@ typedef struct DdeEnumServices {
     HWND hwnd;
 } DdeEnumServices;
 
-typedef struct ThreadSpecificData {
+typedef struct {
     Conversation *currentConversations;
 				/* A list of conversations currently being
 				 * processed. */
@@ -97,10 +79,10 @@ static DWORD ddeInstance;	/* The application instance handle given to us
 				 * by DdeInitialize. */
 static int ddeIsServer = 0;
 
-#define TCL_DDE_VERSION		"1.4.0"
+#define TCL_DDE_VERSION		"1.4.2"
 #define TCL_DDE_PACKAGE_NAME	"dde"
-#define TCL_DDE_SERVICE_NAME	TEXT("TclEval")
-#define TCL_DDE_EXECUTE_RESULT	TEXT("$TCLEVAL$EXECUTE$RESULT")
+#define TCL_DDE_SERVICE_NAME	L"TclEval"
+#define TCL_DDE_EXECUTE_RESULT	L"$TCLEVAL$EXECUTE$RESULT"
 
 #define DDE_FLAG_ASYNC 1
 #define DDE_FLAG_BINARY 2
@@ -114,29 +96,64 @@ TCL_DECLARE_MUTEX(ddeMutex)
 
 static LRESULT CALLBACK	DdeClientWindowProc(HWND hwnd, UINT uMsg,
 			    WPARAM wParam, LPARAM lParam);
-static int		DdeCreateClient(struct DdeEnumServices *es);
+static int		DdeCreateClient(DdeEnumServices *es);
 static BOOL CALLBACK	DdeEnumWindowsCallback(HWND hwndTarget,
 			    LPARAM lParam);
-static void		DdeExitProc(ClientData clientData);
+static void		DdeExitProc(void *clientData);
 static int		DdeGetServicesList(Tcl_Interp *interp,
-			    const TCHAR *serviceName, const TCHAR *topicName);
+			    const WCHAR *serviceName, const WCHAR *topicName);
 static HDDEDATA CALLBACK DdeServerProc(UINT uType, UINT uFmt, HCONV hConv,
 			    HSZ ddeTopic, HSZ ddeItem, HDDEDATA hData,
 			    DWORD dwData1, DWORD dwData2);
 static LRESULT		DdeServicesOnAck(HWND hwnd, WPARAM wParam,
 			    LPARAM lParam);
-static void		DeleteProc(ClientData clientData);
+static void		DeleteProc(void *clientData);
 static Tcl_Obj *	ExecuteRemoteObject(RegisteredInterp *riPtr,
 			    Tcl_Obj *ddeObjectPtr);
 static int		MakeDdeConnection(Tcl_Interp *interp,
-			    const TCHAR *name, HCONV *ddeConvPtr);
+			    const WCHAR *name, HCONV *ddeConvPtr);
 static void		SetDdeError(Tcl_Interp *interp);
-static int		DdeObjCmd(ClientData clientData,
+static int		DdeObjCmd(void *clientData,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *const objv[]);
 
-EXTERN int		Dde_Init(Tcl_Interp *interp);
-EXTERN int		Dde_SafeInit(Tcl_Interp *interp);
+#if (TCL_MAJOR_VERSION < 9) && (TCL_MINOR_VERSION < 7)
+# if TCL_UTF_MAX > 3
+#   define Tcl_WCharToUtfDString(a,b,c) Tcl_WinTCharToUtf((TCHAR *)(a),(b)*sizeof(WCHAR),c)
+#   define Tcl_UtfToWCharDString(a,b,c) (WCHAR *)Tcl_WinUtfToTChar(a,b,c)
+# else
+#   define Tcl_WCharToUtfDString Tcl_UniCharToUtfDString
+#   define Tcl_UtfToWCharDString Tcl_UtfToUniCharDString
+# endif
+#endif
+
+static unsigned char *
+getByteArrayFromObj(
+	Tcl_Obj *objPtr,
+	size_t *lengthPtr
+) {
+    int length;
+
+    unsigned char *result = Tcl_GetByteArrayFromObj(objPtr, &length);
+#if TCL_MAJOR_VERSION > 8
+    if (sizeof(TCL_HASH_TYPE) > sizeof(int)) {
+	/* 64-bit and TIP #494 situation: */
+	 *lengthPtr = *(TCL_HASH_TYPE *) objPtr->internalRep.twoPtrValue.ptr1;
+    } else
+#endif
+	/* 32-bit or without TIP #494 */
+    *lengthPtr = (size_t) (unsigned) length;
+    return result;
+}
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+DLLEXPORT int		Dde_Init(Tcl_Interp *interp);
+DLLEXPORT int		Dde_SafeInit(Tcl_Interp *interp);
+#ifdef __cplusplus
+}
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -158,20 +175,13 @@ int
 Dde_Init(
     Tcl_Interp *interp)
 {
-    if (!Tcl_InitStubs(interp, "8.1", 0)) {
+    if (!Tcl_InitStubs(interp, "8.5", 0)) {
 	return TCL_ERROR;
     }
 
-#ifdef UNICODE
-    if (TclWinGetPlatformId() < VER_PLATFORM_WIN32_NT) {
-	Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		"Win32s and Windows 9x are not supported platforms", -1));
-	return TCL_ERROR;
-    }
-#endif
     Tcl_CreateObjCommand(interp, "dde", DdeObjCmd, NULL, NULL);
     Tcl_CreateExitHandler(DdeExitProc, NULL);
-    return Tcl_PkgProvide(interp, TCL_DDE_PACKAGE_NAME, TCL_DDE_VERSION);
+    return Tcl_PkgProvideEx(interp, TCL_DDE_PACKAGE_NAME, TCL_DDE_VERSION, NULL);
 }
 
 /*
@@ -241,7 +251,7 @@ Initialize(void)
     if (ddeInstance == 0) {
 	Tcl_MutexLock(&ddeMutex);
 	if (ddeInstance == 0) {
-	    if (DdeInitialize(&ddeInstance, (PFNCALLBACK) DdeServerProc,
+	    if (DdeInitializeW(&ddeInstance, (PFNCALLBACK) DdeServerProc,
 		    CBF_SKIP_REGISTRATIONS | CBF_SKIP_UNREGISTRATIONS
 		    | CBF_FAIL_POKES, 0) != DMLERR_NO_ERROR) {
 		ddeInstance = 0;
@@ -254,7 +264,7 @@ Initialize(void)
 	if ((ddeServiceGlobal == 0) && (nameFound != 0)) {
 	    ddeIsServer = 1;
 	    Tcl_CreateExitHandler(DdeExitProc, NULL);
-	    ddeServiceGlobal = DdeCreateStringHandle(ddeInstance,
+	    ddeServiceGlobal = DdeCreateStringHandleW(ddeInstance,
 		    TCL_DDE_SERVICE_NAME, CP_WINUNICODE);
 	    DdeNameService(ddeInstance, ddeServiceGlobal, 0L, DNS_REGISTER);
 	} else {
@@ -289,10 +299,10 @@ Initialize(void)
  *----------------------------------------------------------------------
  */
 
-static const TCHAR *
+static const WCHAR *
 DdeSetServerName(
     Tcl_Interp *interp,
-    const TCHAR *name, /* The name that will be used to refer to the
+    const WCHAR *name, /* The name that will be used to refer to the
 				 * interpreter in later "send" commands. Must
 				 * be globally unique. */
     int flags,		/* DDE_FLAG_FORCE or 0 */
@@ -302,7 +312,7 @@ DdeSetServerName(
     int suffix, offset;
     RegisteredInterp *riPtr, *prevPtr;
     Tcl_DString dString;
-    const TCHAR *actualName;
+    const WCHAR *actualName;
     Tcl_Obj *srvListPtr = NULL, **srvPtrPtr = NULL;
     int n, srvCount = 0, lastSuffix, r = TCL_OK;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -340,7 +350,7 @@ DdeSetServerName(
 	 * current interp, but it doesn't have a name.
 	 */
 
-	return TEXT("");
+	return L"";
     }
 
     /*
@@ -361,8 +371,8 @@ DdeSetServerName(
 		    &srvPtrPtr);
 	}
 	if (r != TCL_OK) {
-	    Tcl_WinUtfToTChar(Tcl_GetStringResult(interp), -1, &dString);
-	    OutputDebugString((TCHAR *) Tcl_DStringValue(&dString));
+	    Tcl_DStringInit(&dString);
+	    OutputDebugStringW(Tcl_UtfToWCharDString(Tcl_GetString(Tcl_GetObjResult(interp)), -1, &dString));
 	    Tcl_DStringFree(&dString);
 	    return NULL;
 	}
@@ -380,14 +390,14 @@ DdeSetServerName(
 	    lastSuffix = suffix;
 	    if (suffix > 1) {
 		if (suffix == 2) {
-		    Tcl_DStringAppend(&dString, (char *)name, _tcslen(name) * sizeof(TCHAR));
-		    Tcl_DStringAppend(&dString, (char *)TEXT(" #"), 2 * sizeof(TCHAR));
+		    Tcl_DStringAppend(&dString, (char *)name, wcslen(name) * sizeof(WCHAR));
+		    Tcl_DStringAppend(&dString, (char *)L" #", 2 * sizeof(WCHAR));
 		    offset = Tcl_DStringLength(&dString);
-		    Tcl_DStringSetLength(&dString, offset + sizeof(TCHAR) * TCL_INTEGER_SPACE);
-		    actualName = (TCHAR *) Tcl_DStringValue(&dString);
+		    Tcl_DStringSetLength(&dString, offset + sizeof(WCHAR) * TCL_INTEGER_SPACE);
+		    actualName = (WCHAR *) Tcl_DStringValue(&dString);
 		}
-		_sntprintf((TCHAR *) (Tcl_DStringValue(&dString) + offset),
-			TCL_INTEGER_SPACE, TEXT("%d"), suffix);
+		_snwprintf((WCHAR *) (Tcl_DStringValue(&dString) + offset),
+			TCL_INTEGER_SPACE, L"%d", suffix);
 	    }
 
 	    /*
@@ -399,8 +409,9 @@ DdeSetServerName(
 		Tcl_DString ds;
 
 		Tcl_ListObjIndex(interp, srvPtrPtr[n], 1, &namePtr);
-	    Tcl_WinUtfToTChar(Tcl_GetString(namePtr), -1, &ds);
-		if (_tcscmp(actualName, (TCHAR *)Tcl_DStringValue(&ds)) == 0) {
+		Tcl_DStringInit(&ds);
+		Tcl_UtfToWCharDString(Tcl_GetString(namePtr), -1, &ds);
+		if (wcscmp(actualName, (WCHAR *)Tcl_DStringValue(&ds)) == 0) {
 		    suffix++;
 		    Tcl_DStringFree(&ds);
 		    break;
@@ -414,16 +425,16 @@ DdeSetServerName(
      * We have found a unique name. Now add it to the registry.
      */
 
-    riPtr = ckalloc(sizeof(RegisteredInterp));
+    riPtr = (RegisteredInterp *) Tcl_Alloc(sizeof(RegisteredInterp));
     riPtr->interp = interp;
-    riPtr->name = ckalloc((_tcslen(actualName) + 1) * sizeof(TCHAR));
+    riPtr->name = (WCHAR *) Tcl_Alloc((wcslen(actualName) + 1) * sizeof(WCHAR));
     riPtr->nextPtr = tsdPtr->interpListPtr;
     riPtr->handlerPtr = handlerPtr;
     if (riPtr->handlerPtr != NULL) {
 	Tcl_IncrRefCount(riPtr->handlerPtr);
     }
     tsdPtr->interpListPtr = riPtr;
-    _tcscpy(riPtr->name, actualName);
+    wcscpy(riPtr->name, actualName);
 
     if (Tcl_IsSafe(interp)) {
 	Tcl_ExposeCommand(interp, "dde", "dde");
@@ -495,8 +506,7 @@ DdeGetRegistrationPtr(
 
 static void
 DeleteProc(
-    ClientData clientData)	/* The interp we are deleting passed as
-				 * ClientData. */
+    void *clientData)	/* The interp we are deleting. */
 {
     RegisteredInterp *riPtr = (RegisteredInterp *) clientData;
     RegisteredInterp *searchPtr, *prevPtr;
@@ -517,7 +527,7 @@ DeleteProc(
 	    prevPtr->nextPtr = searchPtr->nextPtr;
 	}
     }
-    ckfree(riPtr->name);
+    Tcl_Free((char *) riPtr->name);
     if (riPtr->handlerPtr) {
 	Tcl_DecrRefCount(riPtr->handlerPtr);
     }
@@ -555,7 +565,7 @@ ExecuteRemoteObject(
     Tcl_Obj *returnPackagePtr;
     int result = TCL_OK;
 
-    if (riPtr->handlerPtr == NULL && Tcl_IsSafe(riPtr->interp)) {
+    if ((riPtr->handlerPtr == NULL) && Tcl_IsSafe(riPtr->interp)) {
 	Tcl_SetObjResult(riPtr->interp, Tcl_NewStringObj("permission denied: "
 		"a handler procedure must be defined for use in a safe "
 		"interp", -1));
@@ -633,18 +643,20 @@ DdeServerProc(
     HSZ ddeTopic, HSZ ddeItem,	/* String handles. Transaction-type
 				 * dependent. */
     HDDEDATA hData,		/* DDE data. Transaction-type dependent. */
-    DWORD dwData1, DWORD dwData2)
+    DWORD unused1, DWORD unused2)
 				/* Transaction-dependent data. */
 {
     Tcl_DString dString;
-    int len;
+    size_t len;
     DWORD dlen;
-    TCHAR *utilString;
+    WCHAR *utilString;
     Tcl_Obj *ddeObjectPtr;
     HDDEDATA ddeReturn = NULL;
     RegisteredInterp *riPtr;
     Conversation *convPtr, *prevConvPtr;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    (void)unused1;
+    (void)unused2;
 
     switch(uType) {
     case XTYP_CONNECT:
@@ -653,16 +665,16 @@ DdeServerProc(
 	 * sure we have a valid topic.
 	 */
 
-	len = DdeQueryString(ddeInstance, ddeTopic, NULL, 0, CP_WINUNICODE);
+	len = DdeQueryStringW(ddeInstance, ddeTopic, NULL, 0, CP_WINUNICODE);
 	Tcl_DStringInit(&dString);
-	Tcl_DStringSetLength(&dString, (len + 1) * sizeof(TCHAR) - 1);
-	utilString = (TCHAR *) Tcl_DStringValue(&dString);
-	DdeQueryString(ddeInstance, ddeTopic, utilString, (DWORD) len + 1,
+	Tcl_DStringSetLength(&dString, (len + 1) * sizeof(WCHAR) - 1);
+	utilString = (WCHAR *) Tcl_DStringValue(&dString);
+	DdeQueryStringW(ddeInstance, ddeTopic, utilString, (DWORD) len + 1,
 		CP_WINUNICODE);
 
 	for (riPtr = tsdPtr->interpListPtr; riPtr != NULL;
 		riPtr = riPtr->nextPtr) {
-	    if (_tcsicmp(utilString, riPtr->name) == 0) {
+	    if (_wcsicmp(utilString, riPtr->name) == 0) {
 		Tcl_DStringFree(&dString);
 		return (HDDEDATA) TRUE;
 	    }
@@ -678,16 +690,16 @@ DdeServerProc(
 	 * result to return in an XTYP_REQUEST.
 	 */
 
-	len = DdeQueryString(ddeInstance, ddeTopic, NULL, 0, CP_WINUNICODE);
+	len = DdeQueryStringW(ddeInstance, ddeTopic, NULL, 0, CP_WINUNICODE);
 	Tcl_DStringInit(&dString);
-	Tcl_DStringSetLength(&dString,  (len + 1) * sizeof(TCHAR) - 1);
-	utilString = (TCHAR *) Tcl_DStringValue(&dString);
-	DdeQueryString(ddeInstance, ddeTopic, utilString, (DWORD) len + 1,
+	Tcl_DStringSetLength(&dString,  (len + 1) * sizeof(WCHAR) - 1);
+	utilString = (WCHAR *) Tcl_DStringValue(&dString);
+	DdeQueryStringW(ddeInstance, ddeTopic, utilString, (DWORD) len + 1,
 		CP_WINUNICODE);
 	for (riPtr = tsdPtr->interpListPtr; riPtr != NULL;
 		riPtr = riPtr->nextPtr) {
-	    if (_tcsicmp(riPtr->name, utilString) == 0) {
-		convPtr = ckalloc(sizeof(Conversation));
+	    if (_wcsicmp(riPtr->name, utilString) == 0) {
+		convPtr = (Conversation *) Tcl_Alloc(sizeof(Conversation));
 		convPtr->nextPtr = tsdPtr->currentConversations;
 		convPtr->returnPackagePtr = NULL;
 		convPtr->hConv = hConv;
@@ -717,7 +729,7 @@ DdeServerProc(
 		if (convPtr->returnPackagePtr != NULL) {
 		    Tcl_DecrRefCount(convPtr->returnPackagePtr);
 		}
-		ckfree(convPtr);
+		Tcl_Free((char *) convPtr);
 		break;
 	    }
 	}
@@ -743,22 +755,25 @@ DdeServerProc(
 	}
 
 	if (convPtr != NULL) {
+	    Tcl_DString dsBuf;
 	    char *returnString;
 
-	    len = DdeQueryString(ddeInstance, ddeItem, NULL, 0, CP_WINUNICODE);
+	    len = DdeQueryStringW(ddeInstance, ddeItem, NULL, 0, CP_WINUNICODE);
 	    Tcl_DStringInit(&dString);
-	    Tcl_DStringSetLength(&dString, (len + 1) * sizeof(TCHAR) - 1);
-	    utilString = (TCHAR *) Tcl_DStringValue(&dString);
-	    DdeQueryString(ddeInstance, ddeItem, utilString, (DWORD) len + 1,
+	    Tcl_DStringInit(&dsBuf);
+	    Tcl_DStringSetLength(&dString, (len + 1) * sizeof(WCHAR) - 1);
+	    utilString = (WCHAR *) Tcl_DStringValue(&dString);
+	    DdeQueryStringW(ddeInstance, ddeItem, utilString, (DWORD) len + 1,
 		    CP_WINUNICODE);
-	    if (_tcsicmp(utilString, TCL_DDE_EXECUTE_RESULT) == 0) {
-		if (uFmt == CF_TEXT) {
-		    returnString =
-			    Tcl_GetStringFromObj(convPtr->returnPackagePtr, &len);
-		} else {
-		    returnString = (char *)
-			    Tcl_GetUnicodeFromObj(convPtr->returnPackagePtr, &len);
-		    len = sizeof(TCHAR) * len + 1;
+	    if (_wcsicmp(utilString, TCL_DDE_EXECUTE_RESULT) == 0) {
+		returnString =
+			Tcl_GetString(convPtr->returnPackagePtr);
+		len = convPtr->returnPackagePtr->length;
+		if (uFmt != CF_TEXT) {
+		    Tcl_DStringInit(&dsBuf);
+		    Tcl_UtfToWCharDString(returnString, len, &dsBuf);
+		    returnString = Tcl_DStringValue(&dsBuf);
+		    len = Tcl_DStringLength(&dsBuf) + sizeof(WCHAR) - 1;
 		}
 		ddeReturn = DdeCreateDataHandle(ddeInstance, (BYTE *)returnString,
 			(DWORD) len+1, 0, ddeItem, uFmt, 0);
@@ -768,18 +783,20 @@ DdeServerProc(
 		} else {
 		    Tcl_DString ds;
 		    Tcl_Obj *variableObjPtr;
-		    Tcl_WinTCharToUtf(utilString, -1, &ds);
+
+		    Tcl_DStringInit(&ds);
+		    Tcl_WCharToUtfDString(utilString, wcslen(utilString), &ds);
 		    variableObjPtr = Tcl_GetVar2Ex(
 			    convPtr->riPtr->interp, Tcl_DStringValue(&ds), NULL,
 			    TCL_GLOBAL_ONLY);
 		    if (variableObjPtr != NULL) {
-			if (uFmt == CF_TEXT) {
-			    returnString = Tcl_GetStringFromObj(
-				    variableObjPtr, &len);
-			} else {
-			    returnString = (char *) Tcl_GetUnicodeFromObj(
-				    variableObjPtr, &len);
-			    len = sizeof(TCHAR) * len + 1;
+			returnString = Tcl_GetString(variableObjPtr);
+			len = variableObjPtr->length;
+			if (uFmt != CF_TEXT) {
+			    Tcl_DStringInit(&dsBuf);
+			    Tcl_UtfToWCharDString(returnString, len, &dsBuf);
+			    returnString = Tcl_DStringValue(&dsBuf);
+			    len = Tcl_DStringLength(&dsBuf) + sizeof(WCHAR) - 1;
 			}
 			ddeReturn = DdeCreateDataHandle(ddeInstance,
 				(BYTE *)returnString, (DWORD) len+1, 0, ddeItem,
@@ -790,6 +807,7 @@ DdeServerProc(
 		    Tcl_DStringFree(&ds);
 		}
 	    }
+	    Tcl_DStringFree(&dsBuf);
 	    Tcl_DStringFree(&dString);
 	}
 	return ddeReturn;
@@ -814,26 +832,32 @@ DdeServerProc(
 	}
 
 	if (convPtr && !Tcl_IsSafe(convPtr->riPtr->interp)) {
-	    Tcl_DString ds;
+	    Tcl_DString ds, ds2;
 	    Tcl_Obj *variableObjPtr;
+	    DWORD len2;
 
-	    len = DdeQueryString(ddeInstance, ddeItem, NULL, 0, CP_WINUNICODE);
 	    Tcl_DStringInit(&dString);
-	    Tcl_DStringSetLength(&dString, (len + 1) * sizeof(TCHAR) - 1);
-	    utilString = (TCHAR *) Tcl_DStringValue(&dString);
-	    DdeQueryString(ddeInstance, ddeItem, utilString, (DWORD) len + 1,
+	    Tcl_DStringInit(&ds2);
+	    len = DdeQueryStringW(ddeInstance, ddeItem, NULL, 0, CP_WINUNICODE);
+	    Tcl_DStringSetLength(&dString, (len + 1) * sizeof(WCHAR) - 1);
+	    utilString = (WCHAR *) Tcl_DStringValue(&dString);
+	    DdeQueryStringW(ddeInstance, ddeItem, utilString, (DWORD) len + 1,
 		    CP_WINUNICODE);
-	    Tcl_WinTCharToUtf(utilString, -1, &ds);
-	    utilString = (TCHAR *) DdeAccessData(hData, &dlen);
-	    if (uFmt == CF_TEXT) {
-		variableObjPtr = Tcl_NewStringObj((char *)utilString, -1);
-	    } else {
-		variableObjPtr = Tcl_NewUnicodeObj(utilString, -1);
+	    Tcl_DStringInit(&ds);
+	    Tcl_WCharToUtfDString(utilString, wcslen(utilString), &ds);
+	    utilString = (WCHAR *) DdeAccessData(hData, &len2);
+	    len = len2;
+	    if (uFmt != CF_TEXT) {
+		Tcl_DStringInit(&ds2);
+		Tcl_WCharToUtfDString(utilString, wcslen(utilString), &ds2);
+		utilString = (WCHAR *) Tcl_DStringValue(&ds2);
 	    }
+	    variableObjPtr = Tcl_NewStringObj((char *)utilString, -1);
 
 	    Tcl_SetVar2Ex(convPtr->riPtr->interp, Tcl_DStringValue(&ds), NULL,
 		    variableObjPtr, TCL_GLOBAL_ONLY);
 
+	    Tcl_DStringFree(&ds2);
 	    Tcl_DStringFree(&ds);
 	    Tcl_DStringFree(&dString);
 		ddeReturn = (HDDEDATA) DDE_FACK;
@@ -861,7 +885,7 @@ DdeServerProc(
 	    return (HDDEDATA) DDE_FNOTPROCESSED;
 	}
 
-	utilString = (TCHAR *) DdeAccessData(hData, &dlen);
+	utilString = (WCHAR *) DdeAccessData(hData, &dlen);
 	string = (char *) utilString;
 	if (!dlen) {
 	    /* Empty binary array. */
@@ -874,8 +898,13 @@ DdeServerProc(
 	    ddeObjectPtr = Tcl_NewStringObj(string, dlen);
 	} else {
 	    /* unicode */
-	    dlen >>= 1;
-	    ddeObjectPtr = Tcl_NewUnicodeObj((Tcl_UniChar *)utilString, dlen - 1);
+	    Tcl_DString dsBuf;
+
+	    Tcl_DStringInit(&dsBuf);
+	    Tcl_WCharToUtfDString(utilString, (dlen>>1) - 1, &dsBuf);
+	    ddeObjectPtr = Tcl_NewStringObj(Tcl_DStringValue(&dsBuf),
+		    Tcl_DStringLength(&dsBuf));
+	    Tcl_DStringFree(&dsBuf);
 	}
 	Tcl_IncrRefCount(ddeObjectPtr);
 	DdeUnaccessData(hData);
@@ -927,9 +956,9 @@ DdeServerProc(
 	len = dlen;
 	for (i = 0, riPtr = tsdPtr->interpListPtr; i < numItems;
 		i++, riPtr = riPtr->nextPtr) {
-	    returnPtr[i].hszSvc = DdeCreateStringHandle(ddeInstance,
+	    returnPtr[i].hszSvc = DdeCreateStringHandleW(ddeInstance,
 		    TCL_DDE_SERVICE_NAME, CP_WINUNICODE);
-	    returnPtr[i].hszTopic = DdeCreateStringHandle(ddeInstance,
+	    returnPtr[i].hszTopic = DdeCreateStringHandleW(ddeInstance,
 		    riPtr->name, CP_WINUNICODE);
 	}
 	returnPtr[i].hszSvc = NULL;
@@ -961,8 +990,9 @@ DdeServerProc(
 
 static void
 DdeExitProc(
-    ClientData clientData)	    /* Not used in this handler. */
+    void *dummy)		/* Not used. */
 {
+    (void)dummy;
     DdeNameService(ddeInstance, NULL, 0, DNS_UNREGISTER);
     DdeUninitialize(ddeInstance);
     ddeInstance = 0;
@@ -988,14 +1018,14 @@ DdeExitProc(
 static int
 MakeDdeConnection(
     Tcl_Interp *interp,		/* Used to report errors. */
-    const TCHAR *name,		/* The connection to use. */
+    const WCHAR *name,		/* The connection to use. */
     HCONV *ddeConvPtr)
 {
     HSZ ddeTopic, ddeService;
     HCONV ddeConv;
 
-    ddeService = DdeCreateStringHandle(ddeInstance, TCL_DDE_SERVICE_NAME, CP_WINUNICODE);
-    ddeTopic = DdeCreateStringHandle(ddeInstance, name, CP_WINUNICODE);
+    ddeService = DdeCreateStringHandleW(ddeInstance, TCL_DDE_SERVICE_NAME, CP_WINUNICODE);
+    ddeTopic = DdeCreateStringHandleW(ddeInstance, name, CP_WINUNICODE);
 
     ddeConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
     DdeFreeStringHandle(ddeInstance, ddeService);
@@ -1005,7 +1035,8 @@ MakeDdeConnection(
 	if (interp != NULL) {
 	    Tcl_DString dString;
 
-	    Tcl_WinTCharToUtf(name, -1, &dString);
+	    Tcl_DStringInit(&dString);
+	    Tcl_WCharToUtfDString(name, wcslen(name), &dString);
 	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
 		    "no registered server named \"%s\"", Tcl_DStringValue(&dString)));
 	    Tcl_DStringFree(&dString);
@@ -1040,24 +1071,24 @@ MakeDdeConnection(
 
 static int
 DdeCreateClient(
-    struct DdeEnumServices *es)
+    DdeEnumServices *es)
 {
-    WNDCLASSEX wc;
-    static const TCHAR *szDdeClientClassName = TEXT("TclEval client class");
-    static const TCHAR *szDdeClientWindowName = TEXT("TclEval client window");
+    WNDCLASSEXW wc;
+    static const WCHAR *szDdeClientClassName = L"TclEval client class";
+    static const WCHAR *szDdeClientWindowName = L"TclEval client window";
 
     memset(&wc, 0, sizeof(wc));
     wc.cbSize = sizeof(wc);
     wc.lpfnWndProc = DdeClientWindowProc;
     wc.lpszClassName = szDdeClientClassName;
-    wc.cbWndExtra = sizeof(struct DdeEnumServices *);
+    wc.cbWndExtra = sizeof(DdeEnumServices *);
 
     /*
      * Register and create the callback window.
      */
 
-    RegisterClassEx(&wc);
-    es->hwnd = CreateWindowEx(0, szDdeClientClassName, szDdeClientWindowName,
+    RegisterClassExW(&wc);
+    es->hwnd = CreateWindowExW(0, szDdeClientClassName, szDdeClientWindowName,
 	    WS_POPUP, 0, 0, 0, 0, NULL, NULL, NULL, (LPVOID)es);
     return TCL_OK;
 }
@@ -1072,20 +1103,20 @@ DdeClientWindowProc(
     switch (uMsg) {
     case WM_CREATE: {
 	LPCREATESTRUCT lpcs = (LPCREATESTRUCT) lParam;
-	struct DdeEnumServices *es =
-		(struct DdeEnumServices *) lpcs->lpCreateParams;
+	DdeEnumServices *es =
+		(DdeEnumServices *) lpcs->lpCreateParams;
 
 #ifdef _WIN64
-	SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR) es);
+	SetWindowLongPtrW(hwnd, GWLP_USERDATA, (LONG_PTR) es);
 #else
-	SetWindowLong(hwnd, GWL_USERDATA, (LONG) es);
+	SetWindowLongW(hwnd, GWL_USERDATA, (LONG) es);
 #endif
 	return (LRESULT) 0L;
     }
     case WM_DDE_ACK:
 	return DdeServicesOnAck(hwnd, wParam, lParam);
     default:
-	return DefWindowProc(hwnd, uMsg, wParam, lParam);
+	return DefWindowProcW(hwnd, uMsg, wParam, lParam);
     }
 }
 
@@ -1098,27 +1129,29 @@ DdeServicesOnAck(
     HWND hwndRemote = (HWND)wParam;
     ATOM service = (ATOM)LOWORD(lParam);
     ATOM topic = (ATOM)HIWORD(lParam);
-    struct DdeEnumServices *es;
-    TCHAR sz[255];
+    DdeEnumServices *es;
+    WCHAR sz[255];
     Tcl_DString dString;
 
 #ifdef _WIN64
-    es = (struct DdeEnumServices *) GetWindowLongPtr(hwnd, GWLP_USERDATA);
+    es = (DdeEnumServices *) GetWindowLongPtrW(hwnd, GWLP_USERDATA);
 #else
-    es = (struct DdeEnumServices *) GetWindowLong(hwnd, GWL_USERDATA);
+    es = (DdeEnumServices *) GetWindowLongW(hwnd, GWL_USERDATA);
 #endif
 
-    if ((es->service == (ATOM)0 || es->service == service)
-	    && (es->topic == (ATOM)0 || es->topic == topic)) {
+    if (((es->service == (ATOM)0) || (es->service == service))
+	    && ((es->topic == (ATOM)0) || (es->topic == topic))) {
 	Tcl_Obj *matchPtr = Tcl_NewListObj(0, NULL);
 	Tcl_Obj *resultPtr = Tcl_GetObjResult(es->interp);
 
-	GlobalGetAtomName(service, sz, 255);
-	Tcl_WinTCharToUtf(sz, -1, &dString);
+	GlobalGetAtomNameW(service, sz, 255);
+	Tcl_DStringInit(&dString);
+	Tcl_WCharToUtfDString(sz, wcslen(sz), &dString);
 	Tcl_ListObjAppendElement(NULL, matchPtr, Tcl_NewStringObj(Tcl_DStringValue(&dString), -1));
 	Tcl_DStringFree(&dString);
-	GlobalGetAtomName(topic, sz, 255);
-	Tcl_WinTCharToUtf(sz, -1, &dString);
+	GlobalGetAtomNameW(topic, sz, 255);
+	Tcl_DStringInit(&dString);
+	Tcl_WCharToUtfDString(sz, wcslen(sz), &dString);
 	Tcl_ListObjAppendElement(NULL, matchPtr, Tcl_NewStringObj(Tcl_DStringValue(&dString), -1));
 	Tcl_DStringFree(&dString);
 
@@ -1146,7 +1179,7 @@ DdeServicesOnAck(
      * Tell the server we are no longer interested.
      */
 
-    PostMessage(hwndRemote, WM_DDE_TERMINATE, (WPARAM)hwnd, 0L);
+    PostMessageW(hwndRemote, WM_DDE_TERMINATE, (WPARAM)hwnd, 0L);
     return 0L;
 }
 
@@ -1156,9 +1189,9 @@ DdeEnumWindowsCallback(
     LPARAM lParam)
 {
     DWORD_PTR dwResult = 0;
-    struct DdeEnumServices *es = (struct DdeEnumServices *) lParam;
+    DdeEnumServices *es = (DdeEnumServices *) lParam;
 
-    SendMessageTimeout(hwndTarget, WM_DDE_INITIATE, (WPARAM)es->hwnd,
+    SendMessageTimeoutW(hwndTarget, WM_DDE_INITIATE, (WPARAM)es->hwnd,
 	    MAKELONG(es->service, es->topic), SMTO_ABORTIFHUNG, 1000,
 	    &dwResult);
     return TRUE;
@@ -1167,16 +1200,16 @@ DdeEnumWindowsCallback(
 static int
 DdeGetServicesList(
     Tcl_Interp *interp,
-    const TCHAR *serviceName,
-    const TCHAR *topicName)
+    const WCHAR *serviceName,
+    const WCHAR *topicName)
 {
-    struct DdeEnumServices es;
+    DdeEnumServices es;
 
     es.interp = interp;
     es.result = TCL_OK;
     es.service = (serviceName == NULL)
-	    ? (ATOM)0 : GlobalAddAtom(serviceName);
-    es.topic = (topicName == NULL) ? (ATOM)0 : GlobalAddAtom(topicName);
+	    ? (ATOM)0 : GlobalAddAtomW(serviceName);
+    es.topic = (topicName == NULL) ? (ATOM)0 : GlobalAddAtomW(topicName);
 
     Tcl_ResetResult(interp); /* our list is to be appended to result. */
     DdeCreateClient(&es);
@@ -1260,7 +1293,7 @@ SetDdeError(
 
 static int
 DdeObjCmd(
-    ClientData clientData,	/* Used only for deletion */
+    void *dummy,	/* Not used. */
     Tcl_Interp *interp,		/* The interp we are sending from */
     int objc,			/* Number of arguments */
     Tcl_Obj *const *objv)	/* The arguments */
@@ -1291,15 +1324,18 @@ DdeObjCmd(
 	"-binary", NULL
     };
 
-    int index, i, length, argIndex;
+    int index, i, argIndex;
+    size_t length;
     int flags = 0, result = TCL_OK, firstArg = 0;
     HSZ ddeService = NULL, ddeTopic = NULL, ddeItem = NULL, ddeCookie = NULL;
     HDDEDATA ddeData = NULL, ddeItemData = NULL, ddeReturn;
     HCONV hConv = NULL;
-    const TCHAR *serviceName = NULL, *topicName = NULL;
+    const WCHAR *serviceName = NULL, *topicName = NULL;
     const char *string;
     DWORD ddeResult;
     Tcl_Obj *objPtr, *handlerPtr = NULL;
+    Tcl_DString serviceBuf, topicBuf, itemBuf;
+    (void)dummy;
 
     /*
      * Initialize DDE server/client
@@ -1315,6 +1351,9 @@ DdeObjCmd(
 	return TCL_ERROR;
     }
 
+    Tcl_DStringInit(&serviceBuf);
+    Tcl_DStringInit(&topicBuf);
+    Tcl_DStringInit(&itemBuf);
     switch ((enum DdeSubcommands) index) {
     case DDE_SERVERNAME:
 	for (i = 2; i < objc; i++) {
@@ -1364,7 +1403,7 @@ DdeObjCmd(
 	if (objc == 5) {
 	    firstArg = 2;
 	    break;
-	} else if (objc >= 6 && objc <= 7) {
+	} else if ((objc >= 6) && (objc <= 7)) {
 	    firstArg = objc - 3;
 	    for (i = 2; i < firstArg; i++) {
 		if (Tcl_GetIndexFromObj(interp, objv[i], ddeExecOptions,
@@ -1449,11 +1488,13 @@ DdeObjCmd(
     Initialize();
 
     if (firstArg != 1) {
-#ifdef UNICODE
-	serviceName = Tcl_GetUnicodeFromObj(objv[firstArg], &length);
-#else
-	serviceName = Tcl_GetStringFromObj(objv[firstArg], &length);
-#endif
+	const char *src = Tcl_GetString(objv[firstArg]);
+
+	length = objv[firstArg]->length;
+	Tcl_DStringInit(&serviceBuf);
+	Tcl_UtfToWCharDString(src, length, &serviceBuf);
+	serviceName = (WCHAR *) Tcl_DStringValue(&serviceBuf);
+	length = Tcl_DStringLength(&serviceBuf) / sizeof(WCHAR);
     } else {
 	length = 0;
     }
@@ -1461,20 +1502,21 @@ DdeObjCmd(
     if (length == 0) {
 	serviceName = NULL;
     } else if ((index != DDE_SERVERNAME) && (index != DDE_EVAL)) {
-	ddeService = DdeCreateStringHandle(ddeInstance, (void *) serviceName,
+	ddeService = DdeCreateStringHandleW(ddeInstance, serviceName,
 		CP_WINUNICODE);
     }
 
     if ((index != DDE_SERVERNAME) && (index != DDE_EVAL)) {
-#ifdef UNICODE
-	topicName = (TCHAR *) Tcl_GetUnicodeFromObj(objv[firstArg + 1], &length);
-#else
-	topicName = Tcl_GetStringFromObj(objv[firstArg + 1], &length);
-#endif
+	const char *src = Tcl_GetString(objv[firstArg + 1]);
+
+	length = objv[firstArg + 1]->length;
+	Tcl_DStringInit(&topicBuf);
+	topicName = Tcl_UtfToWCharDString(src, length, &topicBuf);
+	length = Tcl_DStringLength(&topicBuf) / sizeof(WCHAR);
 	if (length == 0) {
 	    topicName = NULL;
 	} else {
-	    ddeTopic = DdeCreateStringHandle(ddeInstance, (void *) topicName,
+	    ddeTopic = DdeCreateStringHandleW(ddeInstance, topicName,
 		    CP_WINUNICODE);
 	}
     }
@@ -1484,32 +1526,42 @@ DdeObjCmd(
 	serviceName = DdeSetServerName(interp, serviceName, flags,
 		handlerPtr);
 	if (serviceName != NULL) {
-#ifdef UNICODE
-	    Tcl_SetObjResult(interp, Tcl_NewUnicodeObj((Tcl_UniChar *) serviceName, -1));
-#else
-	    Tcl_SetObjResult(interp, Tcl_NewStringObj(serviceName, -1));
-#endif
+	    Tcl_DString dsBuf;
+
+	    Tcl_DStringInit(&dsBuf);
+	    Tcl_WCharToUtfDString(serviceName, wcslen(serviceName), &dsBuf);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(Tcl_DStringValue(&dsBuf),
+		    Tcl_DStringLength(&dsBuf)));
+	    Tcl_DStringFree(&dsBuf);
 	} else {
 	    Tcl_ResetResult(interp);
 	}
 	break;
 
     case DDE_EXECUTE: {
-	int dataLength;
-	const Tcl_UniChar *dataString;
+	size_t dataLength;
+	const void *dataString;
+	Tcl_DString dsBuf;
 
+	Tcl_DStringInit(&dsBuf);
 	if (flags & DDE_FLAG_BINARY) {
-	    dataString = (const Tcl_UniChar *)
-		    Tcl_GetByteArrayFromObj(objv[firstArg + 2], &dataLength);
-	} else {
 	    dataString =
-		    Tcl_GetUnicodeFromObj(objv[firstArg + 2], &dataLength);
-	    dataLength = (dataLength + 1) * sizeof(Tcl_UniChar);
+		    getByteArrayFromObj(objv[firstArg + 2], &dataLength);
+	} else {
+	    const char *src;
+
+	    src = Tcl_GetString(objv[firstArg + 2]);
+	    dataLength = objv[firstArg + 2]->length;
+	    Tcl_DStringInit(&dsBuf);
+	    dataString =
+		    Tcl_UtfToWCharDString(src, dataLength, &dsBuf);
+	    dataLength = Tcl_DStringLength(&dsBuf) + sizeof(WCHAR);
 	}
 
-	if (dataLength <= 0) {
+	if (dataLength + 1 < 2) {
 	    Tcl_SetObjResult(interp,
 		    Tcl_NewStringObj("cannot execute null data", -1));
+	    Tcl_DStringFree(&dsBuf);
 	    Tcl_SetErrorCode(interp, "TCL", "DDE", "NULL", NULL);
 	    result = TCL_ERROR;
 	    break;
@@ -1519,6 +1571,7 @@ DdeObjCmd(
 	DdeFreeStringHandle(ddeInstance, ddeTopic);
 
 	if (hConv == NULL) {
+	    Tcl_DStringFree(&dsBuf);
 	    SetDdeError(interp);
 	    result = TCL_ERROR;
 	    break;
@@ -1544,16 +1597,18 @@ DdeObjCmd(
 	    SetDdeError(interp);
 	    result = TCL_ERROR;
 	}
+	Tcl_DStringFree(&dsBuf);
 	break;
     }
     case DDE_REQUEST: {
-#ifdef UNICODE
-	const TCHAR *itemString = (TCHAR *) Tcl_GetUnicodeFromObj(objv[firstArg + 2],
-		&length);
-#else
-	const TCHAR *itemString = Tcl_GetStringFromObj(objv[firstArg + 2],
-		&length);
-#endif
+	const WCHAR *itemString;
+	const char *src;
+
+	src = Tcl_GetString(objv[firstArg + 2]);
+	length = objv[firstArg + 2]->length;
+	Tcl_DStringInit(&itemBuf);
+	itemString = Tcl_UtfToWCharDString(src, length, &itemBuf);
+	length = Tcl_DStringLength(&itemBuf) / sizeof(WCHAR);
 
 	if (length == 0) {
 	    Tcl_SetObjResult(interp,
@@ -1571,7 +1626,7 @@ DdeObjCmd(
 	    result = TCL_ERROR;
 	} else {
 	    Tcl_Obj *returnObjPtr;
-	    ddeItem = DdeCreateStringHandle(ddeInstance, (void *) itemString,
+	    ddeItem = DdeCreateStringHandleW(ddeInstance, itemString,
 		    CP_WINUNICODE);
 	    if (ddeItem != NULL) {
 		ddeData = DdeClientTransaction(NULL, 0, hConv, ddeItem,
@@ -1581,18 +1636,24 @@ DdeObjCmd(
 		    result = TCL_ERROR;
 		} else {
 		    DWORD tmp;
-		    const Tcl_UniChar *dataString = (const Tcl_UniChar *) DdeAccessData(ddeData, &tmp);
+		    WCHAR *dataString = (WCHAR *) DdeAccessData(ddeData, &tmp);
 
 		    if (flags & DDE_FLAG_BINARY) {
 			returnObjPtr =
-				Tcl_NewByteArrayObj((BYTE *) dataString, (int) tmp);
+				Tcl_NewByteArrayObj((BYTE *) dataString, tmp);
 		    } else {
-			tmp >>= 1;
-			if (tmp && !dataString[(tmp-1)]) {
-			    --tmp;
+			Tcl_DString dsBuf;
+
+			if ((tmp >= sizeof(WCHAR))
+				&& !dataString[tmp / sizeof(WCHAR) - 1]) {
+			    tmp -= sizeof(WCHAR);
 			}
-			returnObjPtr = Tcl_NewUnicodeObj(dataString,
-				(int) tmp);
+			Tcl_DStringInit(&dsBuf);
+			Tcl_WCharToUtfDString(dataString, tmp>>1, &dsBuf);
+			returnObjPtr =
+			    Tcl_NewStringObj(Tcl_DStringValue(&dsBuf),
+				    Tcl_DStringLength(&dsBuf));
+			Tcl_DStringFree(&dsBuf);
 		    }
 		    DdeUnaccessData(ddeData);
 		    DdeFreeDataHandle(ddeData);
@@ -1603,19 +1664,19 @@ DdeObjCmd(
 		result = TCL_ERROR;
 	    }
 	}
-
 	break;
     }
     case DDE_POKE: {
-#ifdef UNICODE
-	const TCHAR *itemString = (TCHAR *) Tcl_GetUnicodeFromObj(objv[firstArg + 2],
-		&length);
-#else
-	const TCHAR *itemString = Tcl_GetStringFromObj(objv[firstArg + 2],
-		&length);
-#endif
+	Tcl_DString dsBuf;
+	const WCHAR *itemString;
 	BYTE *dataString;
+	const char *src;
 
+	src = Tcl_GetString(objv[firstArg + 2]);
+	length = objv[firstArg + 2]->length;
+	Tcl_DStringInit(&itemBuf);
+	itemString = Tcl_UtfToWCharDString(src, length, &itemBuf);
+	length = Tcl_DStringLength(&itemBuf) / sizeof(WCHAR);
 	if (length == 0) {
 	    Tcl_SetObjResult(interp,
 		    Tcl_NewStringObj("cannot have a null item", -1));
@@ -1623,13 +1684,18 @@ DdeObjCmd(
 	    result = TCL_ERROR;
 	    goto cleanup;
 	}
+	Tcl_DStringInit(&dsBuf);
 	if (flags & DDE_FLAG_BINARY) {
 	    dataString = (BYTE *)
-		    Tcl_GetByteArrayFromObj(objv[firstArg + 3], &length);
+		    getByteArrayFromObj(objv[firstArg + 3], &length);
 	} else {
+	    const char *data =
+		    Tcl_GetString(objv[firstArg + 3]);
+	    length = objv[firstArg + 3]->length;
+	    Tcl_DStringInit(&dsBuf);
 	    dataString = (BYTE *)
-		    Tcl_GetUnicodeFromObj(objv[firstArg + 3], &length);
-	    length = 2 * length + 1;
+		    Tcl_UtfToWCharDString(data, length, &dsBuf);
+	    length = Tcl_DStringLength(&dsBuf) + sizeof(WCHAR);
 	}
 
 	hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
@@ -1640,7 +1706,7 @@ DdeObjCmd(
 	    SetDdeError(interp);
 	    result = TCL_ERROR;
 	} else {
-	    ddeItem = DdeCreateStringHandle(ddeInstance, (void *) itemString,
+	    ddeItem = DdeCreateStringHandleW(ddeInstance, itemString,
 		    CP_WINUNICODE);
 	    if (ddeItem != NULL) {
 		ddeData = DdeClientTransaction(dataString, (DWORD) length,
@@ -1654,6 +1720,7 @@ DdeObjCmd(
 		result = TCL_ERROR;
 	    }
 	}
+	Tcl_DStringFree(&dsBuf);
 	break;
     }
 
@@ -1687,7 +1754,7 @@ DdeObjCmd(
 
 	for (riPtr = tsdPtr->interpListPtr; riPtr != NULL;
 		riPtr = riPtr->nextPtr) {
-	    if (_tcsicmp(serviceName, riPtr->name) == 0) {
+	    if (_wcsicmp(serviceName, riPtr->name) == 0) {
 		break;
 	    }
 	}
@@ -1712,7 +1779,7 @@ DdeObjCmd(
 	     * referring to deallocated objects.
 	     */
 
-	    if (Tcl_IsSafe(riPtr->interp) && riPtr->handlerPtr == NULL) {
+	    if (Tcl_IsSafe(riPtr->interp) && (riPtr->handlerPtr == NULL)) {
 		Tcl_SetObjResult(riPtr->interp, Tcl_NewStringObj(
 			"permission denied: a handler procedure must be"
 			" defined for use in a safe interp", -1));
@@ -1757,8 +1824,7 @@ DdeObjCmd(
 		    objPtr = Tcl_GetVar2Ex(sendInterp, "errorInfo", NULL,
 			    TCL_GLOBAL_ONLY);
 		    if (objPtr) {
-			string = Tcl_GetStringFromObj(objPtr, &length);
-			Tcl_AddObjErrorInfo(interp, string, length);
+			Tcl_AppendObjToErrorInfo(interp, objPtr);
 		    }
 
 		    objPtr = Tcl_GetVar2Ex(sendInterp, "errorCode", NULL,
@@ -1772,6 +1838,8 @@ DdeObjCmd(
 	    Tcl_Release(riPtr);
 	    Tcl_Release(sendInterp);
 	} else {
+	    Tcl_DString dsBuf;
+
 	    /*
 	     * This is a non-local request. Send the script to the server and
 	     * poll it for a result.
@@ -1787,9 +1855,15 @@ DdeObjCmd(
 	    }
 
 	    objPtr = Tcl_ConcatObj(objc, objv);
-	    string = (const char *) Tcl_GetUnicodeFromObj(objPtr, &length);
-	    ddeItemData = DdeCreateDataHandle(ddeInstance,
-		    (BYTE *) string, (DWORD) 2*length+2, 0, 0, CF_UNICODETEXT, 0);
+	    string = Tcl_GetString(objPtr);
+	    length = objPtr->length;
+	    Tcl_DStringInit(&dsBuf);
+	    Tcl_UtfToWCharDString(string, length, &dsBuf);
+	    string = Tcl_DStringValue(&dsBuf);
+	    length = Tcl_DStringLength(&dsBuf) + sizeof(WCHAR);
+	    ddeItemData = DdeCreateDataHandle(ddeInstance, (BYTE *) string,
+		    (DWORD) length, 0, 0, CF_UNICODETEXT, 0);
+	    Tcl_DStringFree(&dsBuf);
 
 	    if (flags & DDE_FLAG_ASYNC) {
 		ddeData = DdeClientTransaction((LPBYTE) ddeItemData,
@@ -1801,7 +1875,7 @@ DdeObjCmd(
 			0xFFFFFFFF, hConv, 0,
 			CF_UNICODETEXT, XTYP_EXECUTE, 30000, NULL);
 		if (ddeData != 0) {
-		    ddeCookie = DdeCreateStringHandle(ddeInstance,
+		    ddeCookie = DdeCreateStringHandleW(ddeInstance,
 			    TCL_DDE_EXECUTE_RESULT, CP_WINUNICODE);
 		    ddeData = DdeClientTransaction(NULL, 0, hConv, ddeCookie,
 			    CF_UNICODETEXT, XTYP_REQUEST, 30000, NULL);
@@ -1818,7 +1892,7 @@ DdeObjCmd(
 
 	    if (!(flags & DDE_FLAG_ASYNC)) {
 		Tcl_Obj *resultPtr;
-		Tcl_UniChar *ddeDataString;
+		WCHAR *ddeDataString;
 
 		/*
 		 * The return handle has a two or four element list in it. The
@@ -1829,13 +1903,18 @@ DdeObjCmd(
 		 * variable "errorInfo".
 		 */
 
-		resultPtr = Tcl_NewObj();
 		length = DdeGetData(ddeData, NULL, 0, 0);
-		ddeDataString = ckalloc(length);
+		ddeDataString = (WCHAR *) Tcl_Alloc(length);
 		DdeGetData(ddeData, (BYTE *) ddeDataString, (DWORD) length, 0);
-		length = (length >> 1) - 1;
-		resultPtr = Tcl_NewUnicodeObj(ddeDataString, length);
-		ckfree(ddeDataString);
+		if (length > sizeof(WCHAR)) {
+		    length -= sizeof(WCHAR);
+		}
+		Tcl_DStringInit(&dsBuf);
+		Tcl_WCharToUtfDString(ddeDataString, length>>1, &dsBuf);
+		resultPtr = Tcl_NewStringObj(Tcl_DStringValue(&dsBuf),
+			Tcl_DStringLength(&dsBuf));
+		Tcl_DStringFree(&dsBuf);
+		Tcl_Free((char *) ddeDataString);
 
 		if (Tcl_ListObjIndex(NULL, resultPtr, 0, &objPtr) != TCL_OK) {
 		    Tcl_DecrRefCount(resultPtr);
@@ -1853,9 +1932,7 @@ DdeObjCmd(
 			Tcl_DecrRefCount(resultPtr);
 			goto invalidServerResponse;
 		    }
-		    length = -1;
-		    string = Tcl_GetStringFromObj(objPtr, &length);
-		    Tcl_AddObjErrorInfo(interp, string, length);
+		    Tcl_AppendObjToErrorInfo(interp, objPtr);
 
 		    Tcl_ListObjIndex(NULL, resultPtr, 2, &objPtr);
 		    Tcl_SetObjErrorCode(interp, objPtr);
@@ -1887,6 +1964,9 @@ DdeObjCmd(
     if (hConv != NULL) {
 	DdeDisconnect(hConv);
     }
+    Tcl_DStringFree(&itemBuf);
+    Tcl_DStringFree(&topicBuf);
+    Tcl_DStringFree(&serviceBuf);
     return result;
 }
 
